import teager

from unittest import TestCase

class SymItem(object):
    def __init__(self, n, p, l):
        self.name = n
        self.path = p
        self.line = l

class Callback(object):
    def __init__(self):
        self.syms = []

    def __call__(self, *args, **kwargs):
        symbol = kwargs['symbol']
        filename = kwargs['filename']
        lineno = kwargs['lineno']

        s = SymItem(symbol, filename, lineno)
        self.syms.append(s)

class TeagerFunctionTests(TestCase):

    def test_single_function_c(self):
        callback = Callback()
        teager.parse("test/files/test_single_function.c", callback)
        self.assertEqual(len(callback.syms), 1)
        self.assertEqual(callback.syms[0].name, "only_one_function") 
        self.assertEqual(callback.syms[0].path, "test/files/test_single_function.c") 
        self.assertEqual(callback.syms[0].line, 1) 

    def test_single_function_cpp(self):
        callback = Callback()
        teager.parse("test/files/test_single_function.cpp", callback)
        self.assertEqual(len(callback.syms), 1)
        self.assertEqual(callback.syms[0].name, "only_one_function") 
        self.assertEqual(callback.syms[0].path, "test/files/test_single_function.cpp") 
        self.assertEqual(callback.syms[0].line, 1)
    
    def test_multiple_functions_c(self):
        callback = Callback()
        teager.parse("test/files/test_multiple_functions.c", callback)
        
        self.assertEqual(len(callback.syms), 4)
        
        self.assertEqual(callback.syms[0].name, "function_one")
        self.assertEqual(callback.syms[0].line, 1)

        self.assertEqual(callback.syms[1].name, "function_two")
        self.assertEqual(callback.syms[1].line, 5)

        self.assertEqual(callback.syms[2].name, "function_three")
        self.assertEqual(callback.syms[2].line, 9)

        self.assertEqual(callback.syms[3].name, "function_four")
        self.assertEqual(callback.syms[3].line, 13)

    def test_multiple_functions_cpp(self):
        callback = Callback()
        teager.parse("test/files/test_multiple_functions.cpp", callback)
        
        self.assertEqual(len(callback.syms), 4)
        
        self.assertEqual(callback.syms[0].name, "function_one")
        self.assertEqual(callback.syms[0].line, 1)

        self.assertEqual(callback.syms[1].name, "function_two")
        self.assertEqual(callback.syms[1].line, 5)

        self.assertEqual(callback.syms[2].name, "function_three")
        self.assertEqual(callback.syms[2].line, 9)

        self.assertEqual(callback.syms[3].name, "function_four")
        self.assertEqual(callback.syms[3].line, 13)

    def test_namespace_handling(self):
        callback = Callback()
        teager.parse("test/files/test_naspace_handling.cpp", callback)
        
        self.assertEqual(len(callback.syms), 4)
        self.assertEqual(callback.syms[0].name, "in_namespace_testing_internal")
        self.assertEqual(callback.syms[1].name, "in_namespace_testing")
        self.assertEqual(callback.syms[2].name, "in_namespace_external")
        self.assertEqual(callback.syms[3].name, "in_namespace_anon")

    def test_exception_safety(self):
        class TeagerTestEx(Exception):
            pass

        def callback(**kwargs):
            raise TeagerTestEx('Its a test')
        
        with self.assertRaises(TeagerTestEx):
            teager.parse("test/files/test_naspace_handling.cpp", callback)

    def test_symbol_type(self):
        types = []
        def callback(*args, **kwargs):
            types.append(kwargs['symtype'])
        teager.parse("test/files/test_symbol_type.cpp", callback)
        self.assertEqual(types[0], teager.TYPE_DECLARATION)
        self.assertEqual(types[1], teager.TYPE_DEFINITION)




#!/usr/bin/env python3

from setuptools import setup, Extension, Command

import os 
import glob


config = {
    'libraries': [ 'clang' ],
    'extra_compile_args': [ '-std=c++11', '-Werror' ],
    'sources': [
        'src/teager.cpp',
        'src/Parser.cpp',
    ],
    'include_dirs': [ 'inc' ],
}

def find_llvm():
    llvm = os.environ.get('PATH_TO_LLVM')
    if llvm:
        print('llvm found: ' + llvm)
        return llvm
    ldpath = os.environ.get('LD_LIBRARY_PATH')
    if ldpath:
        for path in ldpath.split(':'):
            paths = glob.glob(path + '/llvm-*')
            if paths:
                print('llvm found: ' + paths[0])
                return paths[0]
    return None


llvm = find_llvm()
if llvm:
    config['include_dirs'] += [ os.path.join(llvm, 'include') ]
    config['library_dirs'] =  [ os.path.join(llvm, 'lib') ]

teager = Extension('teager', **config)


class TestCommand(Command):
    """
    Execute unit tests
    """
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        import os
        import sys
        import unittest

        # get setup.py directory
        setup_file = sys.modules['__main__'].__file__
        setup_dir = os.path.abspath(os.path.dirname(setup_file))

        # use the default shared TestLoader instance
        test_loader = unittest.defaultTestLoader

        # use the basic test runner that outputs to sys.stderr
        test_runner = unittest.TextTestRunner()

        # automatically discover all tests
        test_suite = test_loader.discover(setup_dir)

        # run the test suite
        test_runner.run(test_suite)


setup(
    name='teager',
    version='0.1.5',
    description='AST traversal powered by libclang',
    long_description=open('README.md').read(),
    author='Jem Tucker',
    author_email='mail@jemtucker.com',
    url='https://github.com/jemtucker/teager',
    ext_modules=[
        teager,
    ],
    cmdclass={ 
        'test': TestCommand,
    }, 
    license='GNUv3',
    python_requires='>=3',
    classifiers={
        # Currently unstable
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Programming Language :: Python :: 3',
    })



